package sun.invoke.util;

import java.util.Iterator;
import java.lang.invoke.MethodType;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.List;

public class BytecodeDescriptor
{
    private BytecodeDescriptor() {
    }
    
    public static List<Class<?>> parseMethod(String s, ClassLoader classLoader) {
        return parseMethod(s, 0, s.length(), classLoader);
    }
    
    static List<Class<?>> parseMethod(String s, int n, int n2, ClassLoader systemClassLoader) {
        if (systemClassLoader == null) {
            systemClassLoader = ClassLoader.getSystemClassLoader();
        }
        int[] array = { n };
        ArrayList o = new ArrayList();
        if (array[0] < n2 && s.charAt(array[0]) == '(') {
            int n3 = 0;
            ++array[n3];
            while (array[0] < n2 && s.charAt(array[0]) != ')') {
                Class<Object> sig = parseSig(s, array, n2, systemClassLoader);
                if (sig == null || sig == Void.TYPE) {
                    parseError(s, "bad argument type");
                }
                o.add(sig);
            }
            int n4 = 0;
            ++array[n4];
        }
        else {
            parseError(s, "not a method type");
        }
        Class<Object> sig2 = parseSig(s, array, n2, systemClassLoader);
        if (sig2 == null || array[0] != n2) {
            parseError(s, "bad return type");
        }
        o.add(sig2);
        return o;
    }
    
    private static void parseError(String str, String str2) {
        throw new IllegalArgumentException("bad signature: " + str + ": " + str2);
    }
    
    private static Class<?> parseSig(String s, int[] array, int n, ClassLoader classLoader) {
        if (array[0] == n) {
            return null;
        }
        int char1 = s.charAt(array[0]++);
        if (char1 == 'L') {
            int n2 = array[0];
            int index = s.indexOf(59, n2);
            if (index < 0) {
                return null;
            }
            array[0] = index + 1;
            String replace = s.substring(n2, index).replace('/', '.');
            try {
                return classLoader.loadClass(replace);
            }
            catch (ClassNotFoundException cause) {
                throw new TypeNotPresentException(replace, cause);
            }
        }
        else if (char1 == '[') {
            Class<Object> componentType = parseSig(s, array, n, classLoader);
            if (componentType != null) {
                componentType = Array.newInstance(componentType, 0).getClass();
                return componentType;
            }
            return componentType;
        }
        else {
            return Wrapper.forBasicType((char)char1).primitiveType();
        }
    }
    
    public static String unparse(Class<?> clazz) {
        StringBuilder sb = new StringBuilder();
        unparseSig(clazz, sb);
        return sb.toString();
    }
    
    public static String unparse(MethodType methodType) {
        return unparseMethod(methodType.returnType(), methodType.parameterList());
    }
    
    public static String unparse(Object o) {
        if ((o instanceof Class)) {
            return unparse((Class)o);
        }
        if ((o instanceof MethodType)) {
            return unparse((MethodType)o);
        }
        return (String)o;
    }
    
    public static String unparseMethod(Class<?> clazz, List<Class<?>> list) {
        StringBuilder sb = new StringBuilder();
        sb.append('(');
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            unparseSig((Class)iterator.next(), sb);
        }
        sb.append(')');
        unparseSig(clazz, sb);
        return sb.toString();
    }
    
    private static void unparseSig(Class<?> clazz, StringBuilder sb) {
        int basicTypeChar = Wrapper.forBasicType(clazz).basicTypeChar();
        if (basicTypeChar != 'L') {
            sb.append((char)basicTypeChar);
        }
        else {
            int n = !clazz.isArray();
            if (!(n == 0)) {
                sb.append('L');
            }
            sb.append(clazz.getName().replace('.', '/'));
            if (!(n == 0)) {
                sb.append(';');
            }
        }
    }
}
